"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ProgressOraPlugin = void 0;
var chalk_1 = __importDefault(require("chalk"));
var webpack_1 = __importDefault(require("webpack"));
var ora_1 = __importDefault(require("ora"));
var ProgressBar_1 = __importDefault(require("./ProgressBar"));
var default_pattern = chalk_1.default.bold('[') + ':percent:%' + chalk_1.default.bold('] ') + chalk_1.default.cyan.bold(':text:');
var ProgressOraPlugin = /** @class */ (function () {
    function ProgressOraPlugin(options) {
        if (options === void 0) { options = {}; }
        this.enable = true;
        this.pattern = options.pattern || default_pattern;
        this.pattern_no_stderr = options.pattern_no_stderr || chalk_1.default.bold('█');
        this.update_render = options.update_render || false;
        this.clear = options.clear || false;
        this.clear_on_update = options.clear_on_update || false;
        this.pb = new ProgressBar_1.default('Generate...', 20, options.pattern_no_stderr);
        this.stderr_check = false;
        this.stream = options.stream || process.stderr;
        var enabled = this.stream && this.stream.isTTY;
        if (!enabled) {
            if (!options.stderr_check) {
                this.enable = false;
            }
            this.stderr_check = true;
        }
        if (options.clear && !this.stderr_check) {
            this.stream.write('\x1Bc');
        }
        else {
            this.stream.write('\n');
        }
        this.text = this.pattern.replace(/\:percent\:/, '0').replace(/\:text\:/, 'build start');
        this.clear_on_update = options.clear_on_update;
        this.spinner = !this.stderr_check ? ora_1.default(options).start() : null;
    }
    ProgressOraPlugin.prototype.apply = function (compiler) {
        var _this = this;
        if (!this.enable)
            return;
        var isRunning = false;
        var startTime = 0;
        return new webpack_1.default.ProgressPlugin(function (percent, msg) {
            var newPercent = Math.ceil(percent * 100);
            if (!_this.stderr_check) {
                if (_this.clear_on_update) {
                    _this.stream.write('\x1Bc');
                }
                if (_this.spinner && _this.pattern)
                    _this.spinner.text = _this.pattern.replace(/\:percent\:/, newPercent.toString()).replace(/\:text\:/, msg);
                if (_this.spinner && _this.update_render) {
                    _this.spinner.render();
                }
            }
            else {
                _this.pb.render({
                    percent: (newPercent / 100).toFixed(4),
                    completed: newPercent,
                    total: 100,
                });
            }
            if (!isRunning) {
                isRunning = true;
                startTime = (new Date()).getTime();
            }
            else if (percent >= 1) {
                var now = (new Date()).getTime();
                var buildTime = (now - startTime) / 1000 + 's';
                if (_this.stderr_check) {
                    _this.stream.write('\n' + chalk_1.default.green('✓') + chalk_1.default.green.bold(' Build completed in ' + buildTime + '\n\n'));
                }
                else {
                    if (_this.spinner)
                        _this.spinner.stop();
                    if (_this.clear) {
                        _this.stream.write('\x1Bc');
                    }
                    if (_this.spinner)
                        _this.spinner.succeed(chalk_1.default.green.bold('Build completed in ' + buildTime + '\n\n'));
                }
                isRunning = false;
            }
        });
    };
    return ProgressOraPlugin;
}());
exports.ProgressOraPlugin = ProgressOraPlugin;
exports.default = ProgressOraPlugin;
